/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "scene.h"

void sceneInit(scene_t *scene, engine_t *engine) {
  scene->engine = engine;
  scene->assetOwner = assetManagerHolderCreate(&engine->assetManager);
  renderListInit(&scene->renderList,
    (int32_t)engine->render.width, (int32_t)engine->render.height
  );
}

void sceneRenderStart(scene_t *scene) {
  renderListResize(&scene->renderList,
    (int32_t)scene->engine->render.width, (int32_t)scene->engine->render.height
  );
}

void sceneRenderEnd(
  scene_t *scene,
  shader_t *shaderRenderList,
  shaderuniform_t listUniformView, shaderuniform_t listUniformProjection,
  shaderuniform_t listUniformModel, shaderuniform_t listUniformTextures[],
  shader_t *shaderBackBuffer,
  shaderuniform_t backUniformView, shaderuniform_t backUniformProjection,
  shaderuniform_t backUniformModel, shaderuniform_t backUniformTexture
) {
  if(shaderRenderList == NULL || shaderBackBuffer == NULL) return;
  renderListRender(&scene->renderList, shaderRenderList,
    listUniformView, listUniformProjection,listUniformModel, listUniformTextures
  );
  renderListAsBackbuffer(&scene->renderList, scene->engine, shaderBackBuffer,
    backUniformView, backUniformProjection, backUniformModel, backUniformTexture
  );
}

void sceneDispose(scene_t *scene) {
  assetManagerHolderRelease(&scene->engine->assetManager, scene->assetOwner);
  renderListDispose(&scene->renderList);
}